



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
pyproj --verbose
IF %ERRORLEVEL% NEQ 0 exit /B 1
pyproj --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
pytest -v -s -k "not (test_get_transform_grid_list__bbox__antimeridian or test_get_transform_grid_list__bbox__out_of_bounds)"
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
